@@ -77,8 +77,7 @@ if test "$1" = "--build-w32"; then
 
     ./configure --enable-maintainer-mode  --prefix=${w32root}  \
             --host=${host} --build=${build} \
-            --with-gpg-error-prefix=${w32root} \
-            --disable-padlock-support
+            --with-gpg-error-prefix=${w32root}
     exit $?
 fi
 # ***** end W32 build script *******
