@@ -1,6 +1,6 @@
 /* pubkey.c  -	pubkey dispatcher
- * Copyright (C) 1998, 1999, 2000, 2002, 2003,
- *               2005, 2007 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2002, 2003, 2005, 
+ *               2007, 2008 Free Software Foundation, Inc.
  *
  * This file is part of Libgcrypt.
  *
@@ -15,8 +15,7 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -24,13 +23,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <assert.h>
 
 #include "g10lib.h"
 #include "mpi.h"
 #include "cipher.h"
 #include "ath.h"
 
+
 static gcry_err_code_t pubkey_decrypt (int algo, gcry_mpi_t *result,
                                        gcry_mpi_t *data, gcry_mpi_t *skey,
                                        int flags);
@@ -41,26 +40,41 @@ static gcry_err_code_t pubkey_verify (int algo, gcry_mpi_t hash,
 				     int (*cmp) (void *, gcry_mpi_t),
                                       void *opaque);
 
+
+/* A dummy extraspec so that we do not need to tests the extraspec
+   field from the module specification against NULL and instead
+   directly test the respective fields of extraspecs.  */
+static pk_extra_spec_t dummy_extra_spec;
+
+
 /* This is the list of the default public-key ciphers included in
-   libgcrypt.  */
+   libgcrypt.  FIPS_ALLOWED indicated whether the algorithm is used in
+   FIPS mode. */
 static struct pubkey_table_entry
 {
   gcry_pk_spec_t *pubkey;
+  pk_extra_spec_t *extraspec;
   unsigned int algorithm;
+  int fips_allowed; 
 } pubkey_table[] =
   {
 #if USE_RSA
-    { &_gcry_pubkey_spec_rsa, GCRY_PK_RSA   },
+    { &_gcry_pubkey_spec_rsa,
+      &_gcry_pubkey_extraspec_rsa,   GCRY_PK_RSA, 1},
 #endif
 #if USE_ELGAMAL
-    { &_gcry_pubkey_spec_elg, GCRY_PK_ELG   },
-    { &_gcry_pubkey_spec_elg, GCRY_PK_ELG_E },
+    { &_gcry_pubkey_spec_elg,
+      &dummy_extra_spec,             GCRY_PK_ELG   },
+    { &_gcry_pubkey_spec_elg,
+      &dummy_extra_spec,             GCRY_PK_ELG_E },
 #endif
 #if USE_DSA
-    { &_gcry_pubkey_spec_dsa, GCRY_PK_DSA   },
+    { &_gcry_pubkey_spec_dsa,
+      &_gcry_pubkey_extraspec_dsa,   GCRY_PK_DSA, 1   },
 #endif
 #if USE_ECC
-    { &_gcry_pubkey_spec_ecdsa, GCRY_PK_ECDSA },
+    { &_gcry_pubkey_spec_ecdsa,
+      &_gcry_pubkey_extraspec_ecdsa, GCRY_PK_ECDSA, 0 },
 #endif
     { NULL, 0 },
   };
@@ -82,7 +96,7 @@ static int default_pubkeys_registered;
       ath_mutex_lock (&pubkeys_registered_lock);   \
       if (! default_pubkeys_registered)            \
         {                                          \
-          gcry_pk_register_default ();         \
+          pk_register_default ();                  \
           default_pubkeys_registered = 1;          \
         }                                          \
       ath_mutex_unlock (&pubkeys_registered_lock); \
@@ -101,6 +115,7 @@ dummy_generate (int algorithm, unsigned int nbits, unsigned long dummy,
   (void)dummy;
   (void)skey;
   (void)retfactors;
+  fips_signal_error ("using dummy public key function");
   return GPG_ERR_NOT_IMPLEMENTED;
 }
 
@@ -109,6 +124,7 @@ dummy_check_secret_key (int algorithm, gcry_mpi_t *skey)
 {
   (void)algorithm;
   (void)skey;
+  fips_signal_error ("using dummy public key function");
   return GPG_ERR_NOT_IMPLEMENTED;
 }
 
@@ -121,6 +137,7 @@ dummy_encrypt (int algorithm, gcry_mpi_t *resarr, gcry_mpi_t data,
   (void)data;
   (void)pkey;
   (void)flags;
+  fips_signal_error ("using dummy public key function");
   return GPG_ERR_NOT_IMPLEMENTED;
 }
 
@@ -133,6 +150,7 @@ dummy_decrypt (int algorithm, gcry_mpi_t *result, gcry_mpi_t *data,
   (void)data;
   (void)skey;
   (void)flags;
+  fips_signal_error ("using dummy public key function");
   return GPG_ERR_NOT_IMPLEMENTED;
 }
 
@@ -144,6 +162,7 @@ dummy_sign (int algorithm, gcry_mpi_t *resarr, gcry_mpi_t data,
   (void)resarr;
   (void)data;
   (void)skey;
+  fips_signal_error ("using dummy public key function");
   return GPG_ERR_NOT_IMPLEMENTED;
 }
 
@@ -158,6 +177,7 @@ dummy_verify (int algorithm, gcry_mpi_t hash, gcry_mpi_t *data,
   (void)pkey;
   (void)cmp;
   (void)opaquev;
+  fips_signal_error ("using dummy public key function");
   return GPG_ERR_NOT_IMPLEMENTED;
 }
 
@@ -166,13 +186,14 @@ dummy_get_nbits (int algorithm, gcry_mpi_t *pkey)
 {
   (void)algorithm;
   (void)pkey;
+  fips_signal_error ("using dummy public key function");
   return 0;
 }
 
 /* Internal function.  Register all the pubkeys included in
    PUBKEY_TABLE.  Returns zero on success or an error code.  */
 static void
-gcry_pk_register_default (void)
+pk_register_default (void)
 {
   gcry_err_code_t err = 0;
   int i;
@@ -191,9 +212,12 @@ gcry_pk_register_default (void)
       pubkey_use_dummy (verify);
       pubkey_use_dummy (get_nbits);
 #undef pubkey_use_dummy
+
       err = _gcry_module_add (&pubkeys_registered,
 			      pubkey_table[i].algorithm,
-			      (void *) pubkey_table[i].pubkey, NULL);
+			      (void *) pubkey_table[i].pubkey, 
+			      (void *) pubkey_table[i].extraspec, 
+                              NULL);
     }
 
   if (err)
@@ -231,16 +255,23 @@ gcry_pk_lookup_name (const char *name)
    PUBKEY.  On success, a new algorithm ID is stored in ALGORITHM_ID
    and a pointer representhing this module is stored in MODULE.  */
 gcry_error_t
-gcry_pk_register (gcry_pk_spec_t *pubkey,
-		  unsigned int *algorithm_id,
-		  gcry_module_t *module)
+_gcry_pk_register (gcry_pk_spec_t *pubkey,
+                   pk_extra_spec_t *extraspec,
+                   unsigned int *algorithm_id,
+                   gcry_module_t *module)
 {
   gcry_err_code_t err = GPG_ERR_NO_ERROR;
   gcry_module_t mod;
 
+  /* We do not support module loading in fips mode.  */
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   ath_mutex_lock (&pubkeys_registered_lock);
   err = _gcry_module_add (&pubkeys_registered, 0,
-			  (void *) pubkey, &mod);
+			  (void *) pubkey, 
+			  (void *)(extraspec? extraspec : &dummy_extra_spec), 
+                          &mod);
   ath_mutex_unlock (&pubkeys_registered_lock);
 
   if (! err)
@@ -498,13 +529,20 @@ pubkey_get_nenc (int algorithm)
 }
 
 
+/* Generate a new public key with algorithm ALGORITHM of size NBITS
+   and return it at SKEY. The use of the arguments QBITS, USE_E,
+   XVALUE and CURVE_NAME depend on the ALGORITHM.  RETFACTOR is used
+   by some algorithms to return certain additional information which
+   are in general not required.
+
+   The function returns the error code number or 0 on success. */
 static gcry_err_code_t
 pubkey_generate (int algorithm, unsigned int nbits, unsigned int qbits,
                  unsigned long use_e, gcry_mpi_t xvalue,
-                 const char *curve_name,
+                 const char *curve_name, unsigned int keygen_flags,
                  gcry_mpi_t *skey, gcry_mpi_t **retfactors)
 {
-  gcry_err_code_t err = GPG_ERR_PUBKEY_ALGO;
+  gcry_err_code_t ec = GPG_ERR_PUBKEY_ALGO;
   gcry_module_t pubkey;
 
   REGISTER_DEFAULT_PUBKEYS;
@@ -513,36 +551,57 @@ pubkey_generate (int algorithm, unsigned int nbits, unsigned int qbits,
   pubkey = _gcry_module_lookup_id (pubkeys_registered, algorithm);
   if (pubkey)
     {
-      /* Hack to pass QBITS to the DSA generation.  */
-      if (qbits && pubkey->spec == &_gcry_pubkey_spec_dsa)
+      pk_extra_spec_t *extraspec = pubkey->extraspec;
+
+      if (keygen_flags && (!extraspec || !extraspec->ext_generate))
         {
-          err = _gcry_dsa_generate2
+          /* A keygen flag has been given but the module does not
+             provide an ext_generate function.  We don't want to
+             ignore such a condition as it might eventually be
+             security sensitive..  */
+          ec = GPG_ERR_INV_FLAG;
+        }
+      else if (qbits && pubkey->spec == &_gcry_pubkey_spec_dsa)
+        {
+          /* Hack to pass QBITS to the DSA generation.  fixme: We
+             should merge this into an ext_generate fucntion. */
+          ec = _gcry_dsa_generate2
             (algorithm, nbits, qbits, 0, skey, retfactors);
         }
 #ifdef USE_ELGAMAL
       else if (xvalue && pubkey->spec == &_gcry_pubkey_spec_elg)
         {
-          err = _gcry_elg_generate_using_x
+          /* Fixme: Merge this into an ext_generate fucntion.  */
+          ec = _gcry_elg_generate_using_x
             (algorithm, nbits, xvalue, skey, retfactors);
         }
 #endif /*USE_ELGAMAL*/
 #ifdef USE_ECC
       else if (curve_name && pubkey->spec == &_gcry_pubkey_spec_ecdsa)
         {
-          err = _gcry_ecc_generate
+          /* Fixme: Merge this into an ext_generate fucntion.  */
+          ec = _gcry_ecc_generate
             (algorithm, nbits, curve_name, skey, retfactors);
         }
 #endif /*USE_ECC*/
+      else if (extraspec && extraspec->ext_generate)
+        {
+          /* Use the extended generate function if available.  */
+          ec = extraspec->ext_generate (algorithm, nbits, use_e,
+                                        keygen_flags,
+                                        skey, retfactors);
+        }
       else
         {
-          err = ((gcry_pk_spec_t *) pubkey->spec)->generate 
+          /* Use the standard generate function.  */
+          ec = ((gcry_pk_spec_t *) pubkey->spec)->generate 
             (algorithm, nbits, use_e, skey, retfactors);
         }
       _gcry_module_release (pubkey);
     }
   ath_mutex_unlock (&pubkeys_registered_lock);
 
-  return err;
+  return ec;
 }
 
 static gcry_err_code_t
@@ -582,7 +641,10 @@ pubkey_encrypt (int algorithm, gcry_mpi_t *resarr, gcry_mpi_t data,
   gcry_err_code_t rc;
   int i;
 
-  if (DBG_CIPHER)
+  /* Note: In fips mode DBG_CIPHER will enver evaluate to true but as
+     an extra failsafe protection we explicitly test for fips mode
+     here. */ 
+  if (DBG_CIPHER && !fips_mode ())
     {
       log_debug ("pubkey_encrypt: algo=%d\n", algorithm);
       for(i = 0; i < pubkey_get_npkey (algorithm); i++)
@@ -604,7 +666,7 @@ pubkey_encrypt (int algorithm, gcry_mpi_t *resarr, gcry_mpi_t data,
  ready:
   ath_mutex_unlock (&pubkeys_registered_lock);
 
-  if (!rc && DBG_CIPHER)
+  if (!rc && DBG_CIPHER && !fips_mode ())
     {
       for(i = 0; i < pubkey_get_nenc (algorithm); i++)
 	log_mpidump("  encr:", resarr[i] );
@@ -630,7 +692,7 @@ pubkey_decrypt (int algorithm, gcry_mpi_t *result, gcry_mpi_t *data,
   int i;
 
   *result = NULL; /* so the caller can always do a mpi_free */
-  if (DBG_CIPHER)
+  if (DBG_CIPHER && !fips_mode ())
     {
       log_debug ("pubkey_decrypt: algo=%d\n", algorithm);
       for(i = 0; i < pubkey_get_nskey (algorithm); i++)
@@ -654,7 +716,7 @@ pubkey_decrypt (int algorithm, gcry_mpi_t *result, gcry_mpi_t *data,
  ready:
   ath_mutex_unlock (&pubkeys_registered_lock);
 
-  if (! rc && DBG_CIPHER)
+  if (!rc && DBG_CIPHER && !fips_mode ())
     log_mpidump (" plain:", *result);
 
   return rc;
@@ -676,7 +738,7 @@ pubkey_sign (int algorithm, gcry_mpi_t *resarr, gcry_mpi_t data,
   gcry_err_code_t rc;
   int i;
 
-  if (DBG_CIPHER)
+  if (DBG_CIPHER && !fips_mode ())
     {
       log_debug ("pubkey_sign: algo=%d\n", algorithm);
       for(i = 0; i < pubkey_get_nskey (algorithm); i++)
@@ -699,7 +761,7 @@ pubkey_sign (int algorithm, gcry_mpi_t *resarr, gcry_mpi_t data,
  ready:
   ath_mutex_unlock (&pubkeys_registered_lock);
 
-  if (! rc && DBG_CIPHER)
+  if (!rc && DBG_CIPHER && !fips_mode ())
     for (i = 0; i < pubkey_get_nsig (algorithm); i++)
       log_mpidump ("   sig:", resarr[i]);
 
@@ -720,7 +782,7 @@ pubkey_verify (int algorithm, gcry_mpi_t hash, gcry_mpi_t *data,
   gcry_err_code_t rc;
   int i;
 
-  if (DBG_CIPHER)
+  if (DBG_CIPHER && !fips_mode ())
     {
       log_debug ("pubkey_verify: algo=%d\n", algorithm);
       for (i = 0; i < pubkey_get_npkey (algorithm); i++)
@@ -816,10 +878,10 @@ sexp_elements_extract_ecc (gcry_sexp_t key_sexp, const char *element_names,
   const char *name;
   gcry_sexp_t list;
 
-  /* Clear the array for easir error cleanup. */
+  /* Clear the array for easier error cleanup. */
   for (name = element_names, idx = 0; *name; name++, idx++)
     elements[idx] = NULL;
-  assert (idx >= 6); /* We know that ECC has at least 6 elements.  */
+  gcry_assert (idx >= 6); /* We know that ECC has at least 6 elements.  */
 
   /* Init the array with the available curve parameters. */
   for (name = element_names, idx = 0; *name && !err; name++, idx++)
@@ -844,6 +906,7 @@ sexp_elements_extract_ecc (gcry_sexp_t key_sexp, const char *element_names,
   list = gcry_sexp_find_token (key_sexp, "curve", 5);
   if (list)
     {
+#if USE_ECC
       char *curve;
       gcry_mpi_t params[6];
 
@@ -868,6 +931,10 @@ sexp_elements_extract_ecc (gcry_sexp_t key_sexp, const char *element_names,
           else
             mpi_free (params[idx]);
         }
+#else /* !USE_ECC */
+      err = GPG_ERR_INV_OBJ; /* "curve" given but ECC not supported. */
+      goto leave;
+#endif /* !USE_ECC */
     }
 
   /* Check that all parameters are known.  */
@@ -1097,7 +1164,7 @@ sexp_to_sig (gcry_sexp_t sexp, gcry_mpi_t **retarray,
  * Take sexp and return an array of MPI as used for our internal decrypt
  * function.
  * s_data = (enc-val
- *           [(flags [pkcs1])
+ *           [(flags [pkcs1])]
  *	      (<algo>
  *		(<param_name1> <mpi>)
  *		...
@@ -1322,7 +1389,8 @@ sexp_data_to_mpi (gcry_sexp_t input, unsigned int nbits, gcry_mpi_t *ret_mpi,
         rc = GPG_ERR_INV_OBJ;
     }
   else if (is_pkcs1 && lvalue && for_encryption)
-    { /* Create pkcs#1 block type 2 padding. */
+    { 
+      /* Create pkcs#1 block type 2 padding. */
       unsigned char *frame = NULL;
       size_t nframe = (nbits+7) / 8;
       const void * value;
@@ -1344,7 +1412,7 @@ sexp_data_to_mpi (gcry_sexp_t input, unsigned int nbits, gcry_mpi_t *ret_mpi,
           frame[n++] = 0;
           frame[n++] = 2; /* block type */
           i = nframe - 3 - valuelen;
-          assert (i > 0);
+          gcry_assert (i > 0);
           p = gcry_random_bytes_secure (i, GCRY_STRONG_RANDOM);
           /* Replace zero bytes by new values. */
           for (;;)
@@ -1379,7 +1447,7 @@ sexp_data_to_mpi (gcry_sexp_t input, unsigned int nbits, gcry_mpi_t *ret_mpi,
           frame[n++] = 0;
           memcpy (frame+n, value, valuelen);
           n += valuelen;
-          assert (n == nframe);
+          gcry_assert (n == nframe);
 
 	  /* FIXME, error checking?  */
           gcry_mpi_scan (ret_mpi, GCRYMPI_FMT_USG, frame, n, &nframe);
@@ -1388,7 +1456,8 @@ sexp_data_to_mpi (gcry_sexp_t input, unsigned int nbits, gcry_mpi_t *ret_mpi,
       gcry_free(frame);
     }
   else if (is_pkcs1 && lhash && !for_encryption)
-    { /* Create pkcs#1 block type 1 padding. */
+    { 
+      /* Create pkcs#1 block type 1 padding. */
       if (gcry_sexp_length (lhash) != 3)
         rc = GPG_ERR_INV_OBJ;
       else if ( !(s=gcry_sexp_nth_data (lhash, 1, &n)) || !n )
@@ -1403,6 +1472,7 @@ sexp_data_to_mpi (gcry_sexp_t input, unsigned int nbits, gcry_mpi_t *ret_mpi,
             { "rmd160", GCRY_MD_RMD160 },
             { "sha384", GCRY_MD_SHA384 },
             { "sha512", GCRY_MD_SHA512 },
+            { "sha224", GCRY_MD_SHA224 },
             { "md2",    GCRY_MD_MD2 },
             { "md4",    GCRY_MD_MD4 },
             { "tiger",  GCRY_MD_TIGER },
@@ -1478,7 +1548,7 @@ sexp_data_to_mpi (gcry_sexp_t input, unsigned int nbits, gcry_mpi_t *ret_mpi,
               frame[n++] = 0;
               frame[n++] = 1; /* block type */
               i = nframe - valuelen - asnlen - 3 ;
-              assert (i > 1);
+              gcry_assert (i > 1);
               memset (frame+n, 0xff, i );
               n += i;
               frame[n++] = 0;
@@ -1486,9 +1556,9 @@ sexp_data_to_mpi (gcry_sexp_t input, unsigned int nbits, gcry_mpi_t *ret_mpi,
               n += asnlen;
               memcpy (frame+n, value, valuelen );
               n += valuelen;
-              assert (n == nframe);
+              gcry_assert (n == nframe);
       
-              /* convert it into an MPI, FIXME: error checking?  */
+              /* Convert it into an MPI.  FIXME: error checking?  */
               gcry_mpi_scan (ret_mpi, GCRYMPI_FMT_USG, frame, n, &nframe);
             }
           
@@ -1541,15 +1611,16 @@ gcry_pk_encrypt (gcry_sexp_t *r_ciph, gcry_sexp_t s_data, gcry_sexp_t s_pkey)
   gcry_pk_spec_t *pubkey = NULL;
   gcry_module_t module = NULL;
 
+  *r_ciph = NULL;
+
   REGISTER_DEFAULT_PUBKEYS;
 
-  *r_ciph = NULL;
-  /* get the key */
+  /* Get the key. */
   rc = sexp_to_key (s_pkey, 0, &pkey, &module);
   if (rc)
     goto leave;
 
-  assert (module);
+  gcry_assert (module);
   pubkey = (gcry_pk_spec_t *) module->spec;
 
   /* If aliases for the algorithm name exists, take the first one
@@ -1609,7 +1680,7 @@ gcry_pk_encrypt (gcry_sexp_t *r_ciph, gcry_sexp_t s_data, gcry_sexp_t s_pkey)
     
     /* And now the ugly part: We don't have a function to pass an
      * array to a format string, so we have to do it this way :-(.  */
-    /* FIXME: There is now such a format spefier, so we can could
+    /* FIXME: There is now such a format specifier, so we can
        change the code to be more clear. */
     arg_list = malloc (nelem * sizeof *arg_list);
     if (!arg_list)
@@ -1684,9 +1755,10 @@ gcry_pk_decrypt (gcry_sexp_t *r_plain, gcry_sexp_t s_data, gcry_sexp_t s_skey)
   gcry_module_t module_enc = NULL, module_key = NULL;
   gcry_pk_spec_t *pubkey = NULL;
 
+  *r_plain = NULL;
+
   REGISTER_DEFAULT_PUBKEYS;
 
-  *r_plain = NULL;
   rc = sexp_to_key (s_skey, 1, &skey, &module_key);
   if (rc)
     goto leave;
@@ -1764,7 +1836,10 @@ gcry_pk_decrypt (gcry_sexp_t *r_plain, gcry_sexp_t s_data, gcry_sexp_t s_skey)
               (<algo>
                 (<param_name1> <mpi>)
                 ...
-                (<param_namen> <mpi>))) 
+                (<param_namen> <mpi>))
+             [(hash algo)]) 
+
+  Note that (hash algo) in R_SIG is not used.
 */
 gcry_error_t
 gcry_pk_sign (gcry_sexp_t *r_sig, gcry_sexp_t s_hash, gcry_sexp_t s_skey)
@@ -1776,14 +1851,15 @@ gcry_pk_sign (gcry_sexp_t *r_sig, gcry_sexp_t s_hash, gcry_sexp_t s_skey)
   int i;
   gcry_err_code_t rc;
 
+  *r_sig = NULL;
+
   REGISTER_DEFAULT_PUBKEYS;
 
-  *r_sig = NULL;
   rc = sexp_to_key (s_skey, 1, &skey, &module);
   if (rc)
     goto leave;
 
-  assert (module);
+  gcry_assert (module);
   pubkey = (gcry_pk_spec_t *) module->spec;
   algo_name = pubkey->aliases? *pubkey->aliases : NULL;
   if (!algo_name || !*algo_name)
@@ -2021,12 +2097,13 @@ gcry_pk_genkey (gcry_sexp_t *r_key, gcry_sexp_t s_parms)
   unsigned int qbits;
   gcry_mpi_t xvalue = NULL;
   char *curve = NULL;
-
-  REGISTER_DEFAULT_PUBKEYS;
+  unsigned int keygen_flags = 0;
 
   skey[0] = NULL;
   *r_key = NULL;
 
+  REGISTER_DEFAULT_PUBKEYS;
+
   list = gcry_sexp_find_token (s_parms, "genkey", 0);
   if (!list)
     {
@@ -2128,7 +2205,7 @@ gcry_pk_genkey (gcry_sexp_t *r_key, gcry_sexp_t s_parms)
         }
     }
 
-  /* Handle the optional "curve" parameter. */
+  /* Parse the optional "curve" parameter. */
   l2 = gcry_sexp_find_token (list, "curve", 0);
   if (l2)
     {
@@ -2142,6 +2219,15 @@ gcry_pk_genkey (gcry_sexp_t *r_key, gcry_sexp_t s_parms)
       l2 = NULL;
     }
 
+  /* Parse the optional "transient-key" flag. */
+  l2 = gcry_sexp_find_token (list, "transient-key", 0);
+  if (l2)
+    {
+      keygen_flags |= PUBKEY_FLAG_TRANSIENT_KEY;
+      gcry_sexp_release (l2);
+      l2 = NULL;
+    }
+
 
   /* Unless a curve name has been given, the "nbits" parameter is
      required.  */
@@ -2173,7 +2259,7 @@ gcry_pk_genkey (gcry_sexp_t *r_key, gcry_sexp_t s_parms)
     nbits = 0;
 
   rc = pubkey_generate (module->mod_id, nbits, qbits, use_e, xvalue,
-                        curve, skey, &factors);
+                        curve, keygen_flags, skey, &factors);
   if (rc)
     goto leave;
 
@@ -2236,7 +2322,7 @@ gcry_pk_genkey (gcry_sexp_t *r_key, gcry_sexp_t s_parms)
         p = stpcpy (p, "))");
       }
     strcpy (p, ")");
-    assert (p - string < needed);
+    gcry_assert (p - string < needed);
 
     while (nelem < DIM (mpis))
       mpis[nelem++] = NULL;
@@ -2260,9 +2346,9 @@ gcry_pk_genkey (gcry_sexp_t *r_key, gcry_sexp_t s_parms)
       free (arg_list);
       if (rc)
 	BUG ();
-      assert (DIM (mpis) == 30); /* Reminder to make sure that the
-                                    array gets increased if new
-                                    parameters are added. */
+      gcry_assert (DIM (mpis) == 30); /* Reminder to make sure that
+                                         the array gets increased if
+                                         new parameters are added. */
     }
     gcry_free (string);
   }
@@ -2346,10 +2432,10 @@ gcry_pk_get_keygrip (gcry_sexp_t key, unsigned char *array)
   gcry_sexp_t list = NULL, l2 = NULL;
   gcry_pk_spec_t *pubkey = NULL;
   gcry_module_t module = NULL;
+  pk_extra_spec_t *extraspec;
   const char *s;
   char *name = NULL;
   int idx;
-  int is_rsa;
   const char *elems;
   gcry_md_hd_t md = NULL;
 
@@ -2383,47 +2469,45 @@ gcry_pk_get_keygrip (gcry_sexp_t key, unsigned char *array)
     goto fail; /* Unknown algorithm.  */
 
   pubkey = (gcry_pk_spec_t *) module->spec;
+  extraspec = module->extraspec;
 
-  /* FIXME, special handling should be implemented by the algorithms,
-     not by the libgcrypt core.  */
-  is_rsa = module->mod_id == GCRY_PK_RSA;
   elems = pubkey->elements_grip;
-  if (! elems)
+  if (!elems)
     goto fail; /* No grip parameter.  */
     
   if (gcry_md_open (&md, GCRY_MD_SHA1, 0))
     goto fail;
 
-  for (idx = 0, s = elems; *s; s++, idx++)
+  if (extraspec && extraspec->comp_keygrip)
     {
-      const char *data;
-      size_t datalen;
-
-      l2 = gcry_sexp_find_token (list, s, 1);
-      if (! l2)
+      /* Module specific method to compute a keygrip.  */
+      if (extraspec->comp_keygrip (md, list))
         goto fail;
-      data = gcry_sexp_nth_data (l2, 1, &datalen);
-      if (! data)
-        goto fail;
-      if (!is_rsa)
-        {
-          char buf[30];
-
-          sprintf (buf, "(1:%c%u:", *s, (unsigned int)datalen);
-          gcry_md_write (md, buf, strlen (buf));
-        }
-  
-      /* PKCS-15 says that for RSA only the modulus should be hashed -
-         however, it is not clear wether this is meant to has the raw
-         bytes assuming this is an unsigned integer or whether the DER
-         required 0 should be prefixed. We hash the raw bytes.  For
-         non-RSA we hash S-expressions. */
-      gcry_md_write (md, data, datalen);
-      gcry_sexp_release (l2);
-      if (!is_rsa)
-        gcry_md_write (md, ")", 1);
     }
-
+  else
+    {
+      /* Generic method to compute a keygrip.  */
+      for (idx = 0, s = elems; *s; s++, idx++)
+        {
+          const char *data;
+          size_t datalen;
+          char buf[30];
+          
+          l2 = gcry_sexp_find_token (list, s, 1);
+          if (! l2)
+            goto fail;
+          data = gcry_sexp_nth_data (l2, 1, &datalen);
+          if (! data)
+            goto fail;
+          
+          snprintf (buf, sizeof buf, "(1:%c%u:", *s, (unsigned int)datalen);
+          gcry_md_write (md, buf, strlen (buf));
+          gcry_md_write (md, data, datalen);
+          gcry_sexp_release (l2);
+          gcry_md_write (md, ")", 1);
+        }
+    }
+  
   if (!array)
     {
       array = gcry_malloc (20);
@@ -2471,14 +2555,16 @@ gcry_pk_ctl (int cmd, void *buffer, size_t buflen)
 }
 
 
-/*
-   Return information about the given algorithm
-   WHAT select the kind of information returned:
+/* Return information about the given algorithm
+
+   WHAT selects the kind of information returned:
+
     GCRYCTL_TEST_ALGO:
         Returns 0 when the specified algorithm is available for use.
         Buffer must be NULL, nbytes  may have the address of a variable
         with the required usage of the algorithm. It may be 0 for don't
         care or a combination of the GCRY_PK_USAGE_xxx flags;
+
     GCRYCTL_GET_ALGO_USAGE:
         Return the usage glafs for the give algo.  An invalid alog
         does return 0.  Disabled algos are ignored here becuase we
@@ -2566,6 +2652,7 @@ gcry_pk_algo_info (int algorithm, int what, void *buffer, size_t *nbytes)
 }
 
 
+/* Explicitly initialize this module.  */
 gcry_err_code_t
 _gcry_pk_init (void)
 {
@@ -2623,6 +2710,46 @@ gcry_pk_list (int *list, int *list_length)
   return err;
 }
 
+
+/* Run the selftests for pubkey algorithm ALGO with optional reporting
+   function REPORT.  */
+gpg_error_t
+_gcry_pk_selftest (int algo, selftest_report_func_t report)
+{
+  gcry_module_t module = NULL;
+  pk_extra_spec_t *extraspec = NULL;
+  gcry_err_code_t ec = 0;
+
+  REGISTER_DEFAULT_PUBKEYS;
+
+  ath_mutex_lock (&pubkeys_registered_lock);
+  module = _gcry_module_lookup_id (pubkeys_registered, algo);
+  if (module && !(module->flags & FLAG_MODULE_DISABLED))
+    extraspec = module->extraspec;
+  ath_mutex_unlock (&pubkeys_registered_lock);
+  if (extraspec && extraspec->selftest)
+    ec = extraspec->selftest (algo, report);
+  else
+    {
+      ec = GPG_ERR_PUBKEY_ALGO;
+      if (report)
+        report ("pubkey", algo, "module", 
+                module && !(module->flags & FLAG_MODULE_DISABLED)?
+                "no selftest available" :
+                module? "algorithm disabled" : "algorithm not found");
+    }
+
+  if (module)
+    {
+      ath_mutex_lock (&pubkeys_registered_lock);
+      _gcry_module_release (module);
+      ath_mutex_unlock (&pubkeys_registered_lock);
+    }
+  return gpg_error (ec);
+}
+
+
+/* This function is only used by ac.c!  */
 gcry_err_code_t
 _gcry_pk_get_elements (int algo, char **enc, char **sig)
 {
