@@ -1,6 +1,6 @@
 /* rsa.c  -  RSA function
  * Copyright (C) 1997, 1998, 1999 by Werner Koch (dd9jn)
- * Copyright (C) 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+ * Copyright (C) 2000, 2001, 2002, 2003, 2008 Free Software Foundation, Inc.
  *
  * This file is part of Libgcrypt.
  *
@@ -53,8 +53,9 @@ typedef struct
 
 
 static void test_keys (RSA_secret_key *sk, unsigned nbits);
-static void generate (RSA_secret_key *sk,
-                      unsigned int nbits, unsigned long use_e);
+static gpg_err_code_t generate (RSA_secret_key *sk,
+                                unsigned int nbits, unsigned long use_e,
+                                int transient_key);
 static int  check_secret_key (RSA_secret_key *sk);
 static void public (gcry_mpi_t output, gcry_mpi_t input, RSA_public_key *skey);
 static void secret (gcry_mpi_t output, gcry_mpi_t input, RSA_secret_key *skey);
@@ -109,10 +110,12 @@ check_exponent (void *arg, gcry_mpi_t a)
  *       = 1 request the use of a "secure" exponent; this is required by some 
  *           specification to be 65537.
  *       > 2 Try starting at this value until a working exponent is found.
+ * TRANSIENT_KEY:  If true, generate the primes using the standard RNG.
  * Returns: 2 structures filled with all needed values
  */
-static void
-generate (RSA_secret_key *sk, unsigned int nbits, unsigned long use_e)
+static gpg_err_code_t
+generate (RSA_secret_key *sk, unsigned int nbits, unsigned long use_e,
+          int transient_key)
 {
   gcry_mpi_t p, q; /* the two primes */
   gcry_mpi_t d;    /* the private key */
@@ -123,8 +126,20 @@ generate (RSA_secret_key *sk, unsigned int nbits, unsigned long use_e)
   gcry_mpi_t phi;  /* helper: (p-1)(q-1) */
   gcry_mpi_t g;
   gcry_mpi_t f;
+  gcry_random_level_t random_level;
 
-  /* make sure that nbits is even so that we generate p, q of equal size */
+  if (fips_mode ())
+  {
+    if (nbits < 1024)
+      return GPG_ERR_INV_VALUE;
+    if (transient_key)
+      return GPG_ERR_INV_VALUE;
+  }
+
+  /* The random quality depends on the transient_key flag.  */
+  random_level = transient_key ? GCRY_STRONG_RANDOM : GCRY_VERY_STRONG_RANDOM;
+
+  /* Make sure that nbits is even so that we generate p, q of equal size. */
   if ( (nbits&1) )
     nbits++; 
 
@@ -162,13 +177,15 @@ generate (RSA_secret_key *sk, unsigned int nbits, unsigned long use_e)
       if (use_e)
         { /* Do an extra test to ensure that the given exponent is
              suitable. */
-          p = _gcry_generate_secret_prime (nbits/2, check_exponent, e);
-          q = _gcry_generate_secret_prime (nbits/2, check_exponent, e);
+          p = _gcry_generate_secret_prime (nbits/2, random_level,
+                                           check_exponent, e);
+          q = _gcry_generate_secret_prime (nbits/2, random_level,
+                                           check_exponent, e);
         }
       else
         { /* We check the exponent later. */
-          p = _gcry_generate_secret_prime (nbits/2, NULL, NULL);
-          q = _gcry_generate_secret_prime (nbits/2, NULL, NULL);
+          p = _gcry_generate_secret_prime (nbits/2, random_level, NULL, NULL);
+          q = _gcry_generate_secret_prime (nbits/2, random_level, NULL, NULL);
         }
       if (mpi_cmp (p, q) > 0 ) /* p shall be smaller than q (for calc of u)*/
         mpi_swap(p,q);
@@ -232,6 +249,8 @@ generate (RSA_secret_key *sk, unsigned int nbits, unsigned long use_e)
 
   /* now we can test our keys (this should never fail!) */
   test_keys( sk, nbits - 64 );
+
+  return 0;
 }
 
 
@@ -436,39 +455,52 @@ rsa_unblind (gcry_mpi_t x, gcry_mpi_t ri, gcry_mpi_t n)
  **************  interface  ******************
  *********************************************/
 
-gcry_err_code_t
-_gcry_rsa_generate (int algo, unsigned int nbits, unsigned long use_e,
-                    gcry_mpi_t *skey, gcry_mpi_t **retfactors)
+static gcry_err_code_t
+rsa_generate (int algo, unsigned int nbits, unsigned long use_e,
+              unsigned int keygen_flags,
+              gcry_mpi_t *skey, gcry_mpi_t **retfactors)
 {
   RSA_secret_key sk;
-  gpg_err_code_t rc;
+  gpg_err_code_t ec;
   int i;
 
   (void)algo;
 
-  generate (&sk, nbits, use_e);
-  skey[0] = sk.n;
-  skey[1] = sk.e;
-  skey[2] = sk.d;
-  skey[3] = sk.p;
-  skey[4] = sk.q;
-  skey[5] = sk.u;
-  
-  /* Make an empty list of factors.  */
-  *retfactors = gcry_calloc ( 1, sizeof **retfactors );
-  if (!*retfactors)
+  ec = generate (&sk, nbits, use_e,
+                 !!(keygen_flags & PUBKEY_FLAG_TRANSIENT_KEY) );
+  if (!ec)
     {
-      rc = gpg_err_code_from_errno (errno);
-      for (i=0; i <= 5; i++)
-        {
-          gcry_mpi_release (skey[i]);
-          skey[i] = NULL;
-        }
-    }
-  else
-    rc = 0;
+      skey[0] = sk.n;
+      skey[1] = sk.e;
+      skey[2] = sk.d;
+      skey[3] = sk.p;
+      skey[4] = sk.q;
+      skey[5] = sk.u;
   
-  return rc;
+      /* Make an empty list of factors.  */
+      *retfactors = gcry_calloc ( 1, sizeof **retfactors );
+      if (!*retfactors)
+        {
+          ec = gpg_err_code_from_syserror ();
+          for (i=0; i <= 5; i++)
+            {
+              gcry_mpi_release (skey[i]);
+              skey[i] = NULL;
+            }
+        }
+      else
+        ec = 0;
+    }
+  
+  return ec;
+}
+
+
+gcry_err_code_t
+_gcry_rsa_generate (int algo, unsigned int nbits, unsigned long use_e,
+                    gcry_mpi_t *skey, gcry_mpi_t **retfactors)
+{
+  return rsa_generate (algo, nbits, use_e, 0, skey, retfactors);
 }
 
 
@@ -537,6 +569,9 @@ _gcry_rsa_decrypt (int algo, gcry_mpi_t *result, gcry_mpi_t *data,
 
   y = gcry_mpi_snew (gcry_mpi_get_nbits (sk.n));
 
+  /* We use blinding by default to mitigate timing attacks which can
+     be practically mounted over the network as shown by Brumley and
+     Boney in 2003.  */ 
   if (! (flags & PUBKEY_FLAG_NO_BLINDING))
     {
       /* Initialize blinding.  */
@@ -627,6 +662,13 @@ _gcry_rsa_verify (int algo, gcry_mpi_t hash, gcry_mpi_t *data, gcry_mpi_t *pkey,
   pk.e = pkey[1];
   result = gcry_mpi_new ( 160 );
   public( result, data[0], &pk );
+#ifdef IS_DEVELOPMENT_VERSION
+  if (DBG_CIPHER)
+    {
+      log_mpidump ("rsa verify result:", result );
+      log_mpidump ("             hash:", hash );
+    }
+#endif /*IS_DEVELOPMENT_VERSION*/
   /*rc = (*cmp)( opaquev, result );*/
   rc = mpi_cmp (result, hash) ? GPG_ERR_BAD_SIGNATURE : GPG_ERR_NO_ERROR;
   gcry_mpi_release (result);
@@ -643,6 +685,96 @@ _gcry_rsa_get_nbits (int algo, gcry_mpi_t *pkey)
   return mpi_get_nbits (pkey[0]);
 }
 
+
+/* Compute a keygrip.  MD is the hash context which we are going to
+   update.  KEYPARAM is an S-expression with the key parameters, this
+   is usually a public key but may also be a secret key.  An example
+   of such an S-expression is:
+
+      (rsa
+        (n #00B...#)
+        (e #010001#))
+        
+   PKCS-15 says that for RSA only the modulus should be hashed -
+   however, it is not clear wether this is meant to use the raw bytes
+   (assuming this is an unsigned integer) or whether the DER required
+   0 should be prefixed.  We hash the raw bytes.  */
+static gpg_err_code_t
+compute_keygrip (gcry_md_hd_t md, gcry_sexp_t keyparam)
+{
+  gcry_sexp_t l1;
+  const char *data;
+  size_t datalen;
+
+  l1 = gcry_sexp_find_token (keyparam, "n", 1);
+  if (!l1)
+    return GPG_ERR_NO_OBJ;
+
+  data = gcry_sexp_nth_data (l1, 1, &datalen);
+  if (!data)
+    {
+      gcry_sexp_release (l1);
+      return GPG_ERR_NO_OBJ;
+    }
+
+  gcry_md_write (md, data, datalen);
+  gcry_sexp_release (l1);
+
+  return 0;
+}
+
+
+
+
+/* 
+     Self-test section.
+ */
+
+
+static gpg_err_code_t
+selftests_rsa (selftest_report_func_t report)
+{
+  const char *what;
+  const char *errtxt;
+  
+  what = "low-level";
+  errtxt = NULL; /*selftest ();*/
+  if (errtxt)
+    goto failed;
+
+  /* FIXME:  need more tests.  */
+
+  return 0; /* Succeeded. */
+
+ failed:
+  if (report)
+    report ("pubkey", GCRY_PK_RSA, what, errtxt);
+  return GPG_ERR_SELFTEST_FAILED;
+}
+
+
+/* Run a full self-test for ALGO and return 0 on success.  */
+static gpg_err_code_t
+run_selftests (int algo, selftest_report_func_t report)
+{
+  gpg_err_code_t ec;
+
+  switch (algo)
+    {
+    case GCRY_PK_RSA:
+      ec = selftests_rsa (report);
+      break;
+    default:
+      ec = GPG_ERR_PUBKEY_ALGO;
+      break;
+        
+    }
+  return ec;
+}
+
+
+
+
 static const char *rsa_names[] =
   {
     "rsa",
@@ -664,3 +796,10 @@ gcry_pk_spec_t _gcry_pubkey_spec_rsa =
     _gcry_rsa_verify,
     _gcry_rsa_get_nbits,
   };
+pk_extra_spec_t _gcry_pubkey_extraspec_rsa = 
+  {
+    run_selftests,
+    rsa_generate,
+    compute_keygrip
+  };
+
