@@ -1,5 +1,5 @@
 /* basic.c  -  basic regression tests
- *	Copyright (C) 2001, 2002, 2003, 2005 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2003, 2005, 2008 Free Software Foundation, Inc.
  *
  * This file is part of Libgcrypt.
  *
@@ -14,8 +14,7 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,6 +49,7 @@ test_spec_pubkey_t;
 
 static int verbose;
 static int error_count;
+static int in_fips_mode;
 
 static void
 fail (const char *format, ...)
@@ -127,6 +127,14 @@ check_cbc_mac_cipher (void)
 
   for (i = 0; i < sizeof (tv) / sizeof (tv[0]); i++)
     {
+      if (gcry_cipher_test_algo (tv[i].algo) && in_fips_mode)
+        {
+          if (verbose)
+            fprintf (stderr, "  algorithm %d not available in fips mode\n",
+                     tv[i].algo);
+          continue;
+        }
+
       err = gcry_cipher_open (&hd,
 			      tv[i].algo,
 			      GCRY_CIPHER_MODE_CBC, GCRY_CIPHER_CBC_MAC);
@@ -945,10 +953,39 @@ check_one_cipher (int algo, int mode, int flags)
       return;
     }
 
-  gcry_cipher_close (hd);
-
   if (memcmp (plain, in, 16))
     fail ("algo %d, mode %d, encrypt-decrypt mismatch\n", algo, mode);
+
+  /* Again, using in-place encryption.  */
+  gcry_cipher_reset (hd);
+
+  memcpy (out, plain, 16);
+  err = gcry_cipher_encrypt (hd, out, 16, NULL, 0);
+  if (err)
+    {
+      fail ("algo %d, mode %d, in-place, gcry_cipher_encrypt failed: %s\n",
+	    algo, mode, gpg_strerror (err));
+      gcry_cipher_close (hd);
+      return;
+    }
+
+  gcry_cipher_reset (hd);
+
+  err = gcry_cipher_decrypt (hd, out, 16, NULL, 0);
+  if (err)
+    {
+      fail ("algo %d, mode %d, in-place, gcry_cipher_decrypt failed: %s\n",
+	    algo, mode, gpg_strerror (err));
+      gcry_cipher_close (hd);
+      return;
+    }
+
+  if (memcmp (plain, out, 16))
+    fail ("algo %d, mode %d, in-place, encrypt-decrypt mismatch\n",algo, mode);
+
+
+  gcry_cipher_close (hd);
+
 }
 
 
@@ -1005,6 +1042,13 @@ check_ciphers (void)
     fprintf (stderr, "Starting Cipher checks.\n");
   for (i = 0; algos[i]; i++)
     {
+      if (gcry_cipher_test_algo (algos[i]) && in_fips_mode)
+        {
+          if (verbose)
+            fprintf (stderr, "  algorithm %d not available in fips mode\n",
+		     algos[i]);
+          continue;
+        }
       if (verbose)
 	fprintf (stderr, "  checking %s [%i]\n",
 		 gcry_cipher_algo_name (algos[i]),
@@ -1020,6 +1064,13 @@ check_ciphers (void)
 
   for (i = 0; algos2[i]; i++)
     {
+      if (gcry_cipher_test_algo (algos[i]) && in_fips_mode)
+        {
+          if (verbose)
+            fprintf (stderr, "  algorithm %d not available in fips mode\n",
+		     algos[i]);
+          continue;
+        }
       if (verbose)
 	fprintf (stderr, "  checking `%s'\n",
 		 gcry_cipher_algo_name (algos2[i]));
@@ -1063,9 +1114,10 @@ check_one_md (int algo, const char *data, int len, const char *expect)
     {				/* hash one million times a "a" */
       char aaa[1000];
 
+      /* Write in odd size chunks so that we test the buffering.  */
       memset (aaa, 'a', 1000);
       for (i = 0; i < 1000; i++)
-	gcry_md_write (hd, aaa, 1000);
+        gcry_md_write (hd, aaa, 1000);
     }
   else
     gcry_md_write (hd, data, len);
@@ -1096,6 +1148,7 @@ check_one_md (int algo, const char *data, int len, const char *expect)
   gcry_md_close (hd2);
 }
 
+
 static void
 check_digests (void)
 {
@@ -1263,11 +1316,19 @@ check_digests (void)
 
   for (i = 0; algos[i].md; i++)
     {
+      if (gcry_md_test_algo (algos[i].md) && in_fips_mode)
+        {
+          if (verbose)
+            fprintf (stderr, "  algorithm %d not available in fips mode\n",
+		     algos[i].md);
+          continue;
+        }
       if (verbose)
 	fprintf (stderr, "  checking %s [%i] for length %zi\n", 
 		 gcry_md_algo_name (algos[i].md),
 		 algos[i].md,
-		 strlen(algos[i].data));
+                 !strcmp (algos[i].data, "!")? 
+                 1000000 : strlen(algos[i].data));
 
       check_one_md (algos[i].md, algos[i].data, strlen (algos[i].data),
 		    algos[i].expect);
@@ -1624,6 +1685,13 @@ check_hmac (void)
 
   for (i = 0; algos[i].md; i++)
     {
+      if (gcry_md_test_algo (algos[i].md) && in_fips_mode)
+        {
+          if (verbose)
+            fprintf (stderr, "  algorithm %d not available in fips mode\n",
+		     algos[i].md);
+          continue;
+        }
       if (verbose)
 	fprintf (stderr, "  checking %s [%i] for length %zi\n", 
 		 gcry_md_algo_name (algos[i].md),
@@ -1792,7 +1860,9 @@ get_keys_new (gcry_sexp_t *pkey, gcry_sexp_t *skey)
   if (verbose)
     fprintf (stderr, "  generating RSA key:");  
   rc = gcry_sexp_new (&key_spec,
-		      "(genkey (rsa (nbits 4:1024)))", 0, 1);
+		      in_fips_mode ? "(genkey (rsa (nbits 4:1024)))"
+                      : "(genkey (rsa (nbits 4:1024)(transient-key)))", 
+                      0, 1);
   if (rc)
     die ("error creating S-expression: %s\n", gpg_strerror (rc));
   rc = gcry_pk_genkey (&key, key_spec);
@@ -1936,7 +2006,16 @@ check_pubkey (void)
     fprintf (stderr, "Starting public key checks.\n");
   for (i = 0; i < sizeof (pubkeys) / sizeof (*pubkeys); i++)
     if (pubkeys[i].id)
-      check_one_pubkey (i, pubkeys[i]);
+      {
+        if (gcry_pk_test_algo (pubkeys[i].id) && in_fips_mode)
+          {
+            if (verbose)
+              fprintf (stderr, "  algorithm %d not available in fips mode\n",
+                       pubkeys[i].id);
+            continue;
+          }
+        check_one_pubkey (i, pubkeys[i]);
+      }
   if (verbose)
     fprintf (stderr, "Completed public key checks.\n");
 
@@ -1944,7 +2023,16 @@ check_pubkey (void)
     fprintf (stderr, "Starting additional public key checks.\n");
   for (i = 0; i < sizeof (pubkeys) / sizeof (*pubkeys); i++)
     if (pubkeys[i].id)
-      check_one_pubkey_new (i);
+      {
+        if (gcry_pk_test_algo (pubkeys[i].id) && in_fips_mode)
+          {
+            if (verbose)
+              fprintf (stderr, "  algorithm %d not available in fips mode\n",
+                       pubkeys[i].id);
+            continue;
+          }
+        check_one_pubkey_new (i);
+      }
   if (verbose)
     fprintf (stderr, "Completed additional public key checks.\n");
 
@@ -1952,13 +2040,44 @@ check_pubkey (void)
 
 int
 main (int argc, char **argv)
-{
+{ 
+  gpg_error_t err;
+  int last_argc = -1;
   int debug = 0;
+  int use_fips = 0;
 
-  if (argc > 1 && !strcmp (argv[1], "--verbose"))
-    verbose = 1;
-  else if (argc > 1 && !strcmp (argv[1], "--debug"))
-    verbose = debug = 1;
+  if (argc)
+    { argc--; argv++; }
+
+  while (argc && last_argc != argc )
+    {
+      last_argc = argc;
+      if (!strcmp (*argv, "--"))
+        {
+          argc--; argv++;
+          break;
+        }
+      else if (!strcmp (*argv, "--verbose"))
+        {
+          verbose++;
+          argc--; argv++;
+        }
+      else if (!strcmp (*argv, "--debug"))
+        {
+          verbose = debug = 1;
+          argc--; argv++;
+        }
+      else if (!strcmp (*argv, "--fips"))
+        {
+          use_fips = 1;
+          argc--; argv++;
+        }
+    }          
+
+  gcry_control (GCRYCTL_SET_VERBOSITY, (int)verbose);
+
+  if (use_fips)
+    gcry_control (GCRYCTL_FORCE_FIPS_MODE, 0);
 
   if (!gcry_check_version (GCRYPT_VERSION))
     die ("version mismatch\n");
@@ -1973,6 +2092,9 @@ main (int argc, char **argv)
   /* No valuable keys are create, so we can speed up our RNG. */
   gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
 
+  if ( gcry_control (GCRYCTL_FIPS_MODE_P, 0) )
+    in_fips_mode = 1;
+
   check_ciphers ();
   check_aes128_cbc_cts_cipher ();
   check_cbc_mac_cipher ();
@@ -1983,6 +2105,56 @@ main (int argc, char **argv)
   check_hmac ();
   check_pubkey ();
 
+  if (in_fips_mode)
+    {
+      /* If we are in fips mode do some more tests. */
+      gcry_md_hd_t md;
+
+      /* First trigger a self-test.  */
+      gcry_control (GCRYCTL_FORCE_FIPS_MODE, 0);
+      if (!gcry_control (GCRYCTL_OPERATIONAL_P, 0))
+        fail ("not in operational state after self-test\n");
+      
+      /* Get us into the error state.  */
+      err = gcry_md_open (&md, GCRY_MD_SHA1, 0);
+      if (err)
+        fail ("failed to open SHA-1 hash context: %s\n", gpg_strerror (err));
+      else
+        {
+          err = gcry_md_enable (md, GCRY_MD_SHA256);
+          if (err)
+            fail ("failed to add SHA-256 hash context: %s\n",
+                  gpg_strerror (err));
+          else
+            {
+              /* gcry_md_get_algo is only defined for a context with
+                 just one digest algorithm.  With our setup it should
+                 put the oibrary intoerror state.  */
+              fputs ("Note: Two lines with error messages follow "
+                     "- this is expected\n", stderr);
+              gcry_md_get_algo (md);
+              gcry_md_close (md);
+              if (gcry_control (GCRYCTL_OPERATIONAL_P, 0))
+                fail ("expected error state but still in operational state\n");
+              else
+                {
+                  /* Now run a self-test and to get back into
+                     operational state.  */
+                  gcry_control (GCRYCTL_FORCE_FIPS_MODE, 0);
+                  if (!gcry_control (GCRYCTL_OPERATIONAL_P, 0))
+                    fail ("did not reach operational after error "
+                          "and self-test\n");
+                }
+            }
+        }
+      
+    }
+  else
+    {
+      /* If in standard mode, run selftests.  */
+      gcry_control (GCRYCTL_FORCE_FIPS_MODE, 0);
+    }
+
   if (verbose)
     fprintf (stderr, "\nAll tests completed. Errors: %i\n", error_count);
 
